@@ -3,14 +3,13 @@ Dummy Salesforce driver that simulates some parts of DB API 2
 
 used by the new Django >= 1.6b2
 """
+from django.utils.six import PY3
 
 # All error types described in DB API 2 are implemented the same way as in
 # Django 1.6, otherwise some exceptions are not correctly reported in it.
 
-SIX_PY3 = False  # Python 2 only
 
-
-class Error(Exception if SIX_PY3 else StandardError):
+class Error(Exception if PY3 else StandardError):
 	pass
 
 
@@ -57,7 +56,7 @@ class Connection(object):
 		pass
 
 	def rollback(self):
-		print "Rollback is not implemented."
+		print("Rollback is not implemented.")
 
 
 def connect(**params):
