@@ -1,4 +1,5 @@
 # Django settings for testrunner project.
+import os
 
 DEBUG = True
 TEMPLATE_DEBUG = DEBUG
@@ -9,8 +10,6 @@ ADMINS = (
 
 PERSON_ACCOUNT_ACTIVATED = False
 
-MANAGERS = ADMINS
-
 DATABASES = {
 	'default': {
 		'ENGINE': 'django.db.backends.sqlite3',
@@ -20,10 +19,10 @@ DATABASES = {
 	# in order to protect private secret values from unintentional committing.
 	'salesforce': {
 		'ENGINE': 'salesforce.backend',
-		"CONSUMER_KEY" : '3MVG9JZ_r.QzrS7icihiIU063YpiIRjEFszVnMCZZmLC5KORG6fhJIxw5ahqgV3lDDCrV..PyA8Jr5JT4JzQk',
-		"CONSUMER_SECRET" : '228684408521557504',
-		'USER': 'djangosalesforce@gmail.com',
-		'PASSWORD': 'Cb6>pd8d-A',
+		"CONSUMER_KEY" : os.environ.get('SF_CONSUMER_KEY', ''),
+		"CONSUMER_SECRET" : os.environ.get('SF_CONSUMER_SECRET', ''),
+		'USER': os.environ.get('SF_USER', ''),
+		'PASSWORD': os.environ.get('SF_PASSWORD', ''),
 		'HOST': 'https://login.salesforce.com',
 	}
 }
@@ -41,8 +40,6 @@ TIME_ZONE = 'America/New_York'
 # http://www.i18nguy.com/unicode/language-identifiers.html
 LANGUAGE_CODE = 'en-us'
 
-SITE_ID = 1
-
 # If you set this to False, Django will make some optimizations so as not
 # to load the internationalization machinery.
 USE_I18N = True
@@ -88,12 +85,7 @@ STATICFILES_FINDERS = (
 # Make this unique, and don't share it with anybody.
 SECRET_KEY = '6$y&o(28l)#o1_2rafojb_&zxi*jnivkv)ygj#!01kt0ypsxe$'
 
-# List of callables that know how to import templates from various sources.
-TEMPLATE_LOADERS = (
-	'django.template.loaders.filesystem.Loader',
-	'django.template.loaders.app_directories.Loader',
-#	  'django.template.loaders.eggs.Loader',
-)
+SITE_ID = 1
 
 MIDDLEWARE_CLASSES = (
 	'django.middleware.common.CommonMiddleware',
@@ -101,6 +93,7 @@ MIDDLEWARE_CLASSES = (
 	'django.middleware.csrf.CsrfViewMiddleware',
 	'django.contrib.auth.middleware.AuthenticationMiddleware',
 	'django.contrib.messages.middleware.MessageMiddleware',
+	'django.middleware.clickjacking.XFrameOptionsMiddleware',
 )
 
 ROOT_URLCONF = 'salesforce.testrunner.urls'
@@ -145,7 +138,8 @@ LOGGING = {
 		},
 		'mail_admins': {
 			'level': 'ERROR',
-			'class': 'django.utils.log.AdminEmailHandler'
+			'class': 'django.utils.log.AdminEmailHandler',
+			'filters': ['require_debug_false'],
 		}
 	},
 	'loggers': {
@@ -164,10 +158,18 @@ LOGGING = {
 			'level': 'INFO',
 			'propagate': True,
 		},
+	},
+	'filters': {
+		'require_debug_false': {
+			"()": "django.utils.log.RequireDebugFalse",
+		}
 	}
 }
 
+IPV4_ONLY = True  # for networks with only IPv4 (default is currently IPv4
+# for compatibily with previous beviour and until IPv6 on SF become stable.)
+
 try:
 	from salesforce.testrunner.local_settings import *
-except ImportError, exp:
+except ImportError:
 	pass
